
# Opens a file as a string and writes back the string to the file at
# the end of the block.
#
# Returns the number of written bytes or +nil+ if the file wasn't
# modified.
#
# Note that the file will even be written back in case the block
# raises an exception.
#
# Mode can either be "b" or "+" and specifies to open the file in
# binary mode (no mapping of the plattform's newlines to "\n" is
# done) or to append to it.
#
# Usage:
#   # Reverse contents of "message"
#   File.open_as_string("message") { |str| str.reverse! }
#
#   # Replace "foo" by "bar" in "binary"
#   File.open_as_string("binary", "b") { |str| str.gsub!("foo", "bar") }
#
def File.open_as_string(name, mode = "")
  unless block_given?
    raise(ArgumentError, "Need to supply block to File.open_as_string")
  end

  if mode.is_a?(Numeric) then
    flag, mode = mode, ""
    mode += "b" if flag & File::Constants::BINARY != 0
    mode += "+" if flag & File::Constants::APPEND != 0
  else
    mode.delete!("^b+")
  end

  str = File.open(name, "r#{mode}") { |file| file.read } rescue ""
  old_str = str.clone

  begin
    yield str
  ensure
    if str != old_str then
      File.open(name, "w#{mode}") { |file| file.write(str) }
    end
  end
end



#  _____         _
# |_   _|__  ___| |_
#   | |/ _ \/ __| __|
#   | |  __/\__ \ |_
#   |_|\___||___/\__|
#
=begin test

  require 'test/unit'

  # FIND TEST DIRECTORY
  paths = File.expand_path(File.dirname(__FILE__)).split('/')
  paths.size.downto(1) do |i|
    f = (paths.slice(0..i)+['test']).join('/')
    $TESTDIR = File.join(f,'FIXTURE') if File.directory?(f)
  end
  raise unless $TESTDIR

  class TC_OPEN_AS_STRING < Test::Unit::TestCase

    def test_open_as_string
      f = File.join( $TESTDIR, 'open_as_string_file' )
      t = 'This is a test'
      File.open( f, 'w' ) { |s| s << t }
      assert( File.exists?( f ) )
      File.open_as_string(f) { |s| s.reverse! }
      s = File.read( f )
      assert_equal( t.reverse, s )
    end

  end

=end
